{% extends "base.html" %}

{% block content %}
  <div class="aside">
    <hgroup>
      <h2>{{ article.title_f|typography }}</h2>
      <div>
        <p>{{ article.subtitle_f|typography }}</p>
        <p class="muted">
          {% if article.authors %}
            {% for author in article.authors %}
              <h4><em><a href="{{ base_url }}auteur/{{ author.slug }}/">{{ author.forname }} {{ author.surname }}</a></em></h4>
            {% endfor %}
          {% endif %}
        </p>
        <br>
        <p class="muted">
          <small>
            <a href="{{ base_url }}blog/#date-{{ article.date }}">{{ article.date.strftime('%d %B %Y') }}</a>
            <small class="muted" aria-hidden="true"> ⁙ </small>
            <a href="{{ base_url }}numero/{{ article.numero.slug }}/">{{ article.numero.title_f }}</a>
            {% if article.keywords %}
              <small class="muted" aria-hidden="true"> ⁙ </small>
              {% for keyword in article.keywords %}
                <a href="{{ base_url }}mot-clef/{{ keyword.slug }}/">#{{ keyword.name }}</a>{%- if not loop.last -%}, {% endif %}
              {% endfor %}
            {% endif %}
          </small>
        </p>
      </div>
    </hgroup>
    <div class="article-body">
      {% if article.abstract_fr %}
        <div class="article-abstract">{{ article.abstract_fr|typography|markdown }}</div>
      {% else %}
        {% if article.abstract_en %}
          <div class="article-abstract" lang="en">{{ article.abstract_en|markdown }}</div>
        {% endif %}
      {% endif %}

      {{ article.content_html }}
    </div>
  </div>
{% endblock content %}
